@@ -5,13 +5,8 @@ import subprocess
 from abc import ABC
 from logging import getLogger
 from pathlib import Path
-from typing import (
-    Match,
-    Pattern,
-    Iterator
-)
+from typing import Match, Pattern, Iterator
 
-from phystool.config import config
 from phystool.helper import ContextIterator
 
 logger = getLogger(__name__)
@@ -19,6 +14,7 @@ logger = getLogger(__name__)
 
 class LogFileMessage(ABC):
     """Helper class that stores log file messages."""
+
     PATTERN: Pattern
     VERBOSE = False
 
@@ -45,18 +41,10 @@ class LogFileMessage(ABC):
     def _get_message(self) -> str:
         """Returns the formatted log message"""
         return (
-            "Context:\n" + "".join(
-                [
-                    f"{i:>2}: {context}"
-                    for i, context in enumerate(self._context)
-                ]
-            )
-            + "\nRegex match groups:\n" + "\n".join(
-                [
-                    f"{i+1:>2}: {m}"
-                    for i, m in enumerate(self._match.groups())
-                ]
-            )
+            "Context:\n"
+            + "".join([f"{i:>2}: {context}" for i, context in enumerate(self._context)])
+            + "\nRegex match groups:\n"
+            + "\n".join([f"{i+1:>2}: {m}" for i, m in enumerate(self._match.groups())])
         )
 
 
@@ -81,8 +69,8 @@ class BadBoxMessage(LogFileMessage):
     def _get_message(self) -> str:
         l1, l2 = (
             (self._match.group(7), self._match.group(7))
-            if self._match.group(7) is not None else
-            (self._match.group(5), self._match.group(6))
+            if self._match.group(7) is not None
+            else (self._match.group(5), self._match.group(6))
         )
         return "[B]: {}-{} {} {}{}".format(
             l1 if l1 else "?",
@@ -95,7 +83,7 @@ class BadBoxMessage(LogFileMessage):
 
 class InfoMessage(LogFileMessage):
     PATTERN = re.compile(
-        r"^((?:La|pdf)TeX3?|Package|Class)(?: (\S+))? [iI]nfo(?: \(([\\]?\w+)\))?: (.*)"  # noqa
+        r"^((?:La|pdf)TeX3?|Package|Class)(?: (\S+))? [iI]nfo(?: \(([\\]?\w+)\))?: (.*)"
     )
     # 0 - Whole match (line)
     # 1 - Type ((?:La|pdf)TeX3?|Package|Class)
@@ -120,7 +108,7 @@ class InfoMessage(LogFileMessage):
 
 class WarningMessage(LogFileMessage):
     PATTERN = re.compile(
-        r"^((?:La|pdf)TeX3?|Package|Class)(?: (\S+))? [wW]arning(?: \(([\\]?\w+)\))?: (.*)"  # noqa
+        r"^((?:La|pdf)TeX3?|Package|Class)(?: (\S+))? [wW]arning(?: \(([\\]?\w+)\))?: (.*)"
     )
     # 0 - Whole match (line)
     # 1 - Type ((?:La|pdf)TeX3?|Package|Class)
@@ -131,23 +119,22 @@ class WarningMessage(LogFileMessage):
     should_recompile: bool
 
     class RecompilationRequired(Exception):
-        ERRNO_MESSAGE = (20, 'Recompilation required')
+        ERRNO_MESSAGE = (20, "Recompilation required")
 
     def _get_message(self) -> str:
         out = "[W{}] {}"
         tmp = self._match.group(4)
-        self.should_recompile = ("rerun" in tmp.lower())
+        self.should_recompile = "rerun" in tmp.lower()
         if package := self._match.group(2):
             self.ignore = (
-                package == 'hyperref'
-                and "Token not allowed in a PDF string" in tmp
+                package == "hyperref" and "Token not allowed in a PDF string" in tmp
             )
             out = out.format(f":{package=}", tmp) + "\n"
             loffset = len(package) + 15
             i = 7
             while line := self._context[i].lstrip():
                 if tmp := line.replace(f"({package})", "").lstrip():
-                    out += loffset*" " + tmp
+                    out += loffset * " " + tmp
                 i += 1
 
             return out
@@ -172,7 +159,7 @@ class WarningMessage(LogFileMessage):
 
 class ErrorMessage(LogFileMessage):
     PATTERN = re.compile(
-        r"^(?:! ?((?:La|pdf)TeX3?|Package|Class)(?: (\S+))? [eE]rror(?: \(([\\]?\w+)\))?: (.*)|! (.*))"  # noqa
+        r"^(?:! ?((?:La|pdf)TeX3?|Package|Class)(?: (\S+))? [eE]rror(?: \(([\\]?\w+)\))?: (.*)|! (.*))"
     )
     # 0 - Whole match (line)
     # 1 - Type ((?:La|pdf)TeX3?|Package|Class)
@@ -209,6 +196,10 @@ class ErrorMessage(LogFileMessage):
         "is unknown and is being ignored.": True,
         "Option clash": True,
         "Unicode character": True,
+        "caption outside float.": True,
+        "Not in outer par mode.": True,
+        "Bad math environment delimiter.": True,
+        r"Lonely \item": True,
     }
     _EXTRA_CONTEXT_NONE = {
         "File ended while scanning": False,
@@ -227,6 +218,8 @@ class ErrorMessage(LogFileMessage):
         "Package PGF Math Error": True,
         "Illegal parameter number in definition": False,
         "Illegal unit of measure": True,
+        r"You can't use `\eqno'": True,
+        r"Use of \??? doesn't match its definition.": True,
     }
 
     def _get_full_error_line(self, out: str, replace: str) -> str:
@@ -243,16 +236,12 @@ class ErrorMessage(LogFileMessage):
         out = "[E{}] {}"
         tmp = self._match.group(4)
         if package := self._match.group(2):
-            out = out.format(
-                f":{package=}",
-                self._get_full_error_line(tmp, package)
-            )
+            out = out.format(f":{package=}", self._get_full_error_line(tmp, package))
             tmp = package
             extra_context = self._EXTRA_CONTEXT_PACKAGE
         elif error_type := self._match.group(1):
             out = out.format(
-                f":{error_type}",
-                self._get_full_error_line(tmp, error_type)
+                f":{error_type}", self._get_full_error_line(tmp, error_type)
             )
             extra_context = self._EXTRA_CONTEXT_LATEX
         else:
@@ -266,7 +255,7 @@ class ErrorMessage(LogFileMessage):
                     i = 7
                     while not self._context[i].startswith("l."):
                         i += 1
-                    out += "\n    " + "    ".join(self._context[i:i+2])
+                    out += "\n    " + "    ".join(self._context[i : i + 2])
                 return out
 
         return super()._get_message()
@@ -282,7 +271,7 @@ class Latex3Message(LogFileMessage):
         if not line:  # to catch long lines
             line = self._context[i]
             i += 1
-        while i < n and self._context[i][0] != '<':
+        while i < n and self._context[i][0] != "<":
             line += self._context[i]
             i += 1
 
@@ -295,23 +284,20 @@ class Latex3Message(LogFileMessage):
         return f"[S:LaTeX3] {line}"
 
 
-class LogMessageList:
-    def __init__(self, klass: type[LogFileMessage]):
+class LogMessageList[T: LogFileMessage]:
+    def __init__(self, klass: type[T]):
         self._klass = klass
-        self._message_list: list[LogFileMessage] = []
+        self._message_list: list[T] = []
 
-    def __getitem__(self, idx: int) -> LogFileMessage:
+    def __getitem__(self, idx: int) -> T:
         return self._message_list[idx]
 
-    def __iter__(self) -> Iterator[LogFileMessage]:
+    def __iter__(self) -> Iterator[T]:
         for message in self._message_list:
             yield message
 
     def add(
-        self,
-        line: str,
-        lines_iterable: ContextIterator[str],
-        tex_file: Path
+        self, line: str, lines_iterable: ContextIterator[str], tex_file: Path
     ) -> bool:
         match = self._klass.PATTERN.match(line)
         if match is None:
@@ -322,7 +308,7 @@ class LogMessageList:
             self._message_list.append(message)
         return True
 
-    def display(self) -> None:
+    def as_log(self) -> None:
         for message in self._message_list:
             logger.info(message)
 
@@ -341,41 +327,43 @@ class LatexLogParser:
 
     Relies on the fact that 'texmf.cnf' has been configured so that filenames
     are written on single line ('max_print_line=1000')
+
+    :param texfile: LaTeX file for which the log should be parsed
+    :param message_types: selection of LogfileMessage to parse and display
     """
+
     # should match the last .tex file of a line
     TEX_FILE_PATTERN = re.compile(r".*\((\/.*\.(tex|cls|sty))[\s)$]")
 
     def __init__(
         self,
         texfile: Path,
-        message_types: list[type[LogFileMessage]] = [
+        message_types: list = [
             BadBoxMessage,
             InfoMessage,
             WarningMessage,
             ErrorMessage,
             Latex3Message,
-        ]
+        ],
     ):
         self._logfile = texfile.with_suffix(".log")
-        self._message_map = {
-            mt: LogMessageList(mt)
-            for mt in message_types
-        }
+        self._message_map = {mt: LogMessageList(mt) for mt in message_types}
 
     def _extract_texfile_name_from_line(self, line: str) -> Path | None:
         if tf := self.TEX_FILE_PATTERN.match(line):
             out = Path(tf.group(1)).resolve()
-            if (
-                out.is_relative_to(Path.home())
-                or out.is_relative_to(Path("/tmp"))
-            ):
+            if out.is_relative_to(Path.home()) or out.is_relative_to(Path("/tmp")):
                 return out
         return None
 
     def process(self) -> None:
-        """Checks each line if it contains a message."""
+        """
+        Main method that parses each line of the LaTeX logfile and checks for
+        a matching pattern. When a pattern is matched, a new message is created
+        and added to the correct LogMessageList according to its type.
+        """
         tex_file = None
-        with self._logfile.open(encoding='utf-8', errors='replace') as lf:
+        with self._logfile.open(encoding="utf-8", errors="replace") as lf:
             lines_iterable = ContextIterator(lf, before=6, after=10)
             for line in lines_iterable:
                 if not line:
@@ -384,31 +372,32 @@ class LatexLogParser:
                 if tmp_file := self._extract_texfile_name_from_line(line):
                     tex_file = tmp_file
                 elif (
-                    tex_file
-                    and "==> Fatal" not in line
-                    and "Emergency" not in line
+                    tex_file and "==> Fatal" not in line and "Emergency" not in line
                 ):  # fully ignore these errors
-                    for messages in self._message_map.values():
-                        if messages.add(line, lines_iterable, tex_file):
+                    for message_list in self._message_map.values():
+                        if message_list.add(line, lines_iterable, tex_file):
                             break
 
     def should_recompile(self) -> bool:
-        for message in self._message_map.get(WarningMessage, []):
+        """True if any WarningMessage is flagged as 'should_recompile'"""
+        for message in self.warning():
             if message.should_recompile:
                 return True
         return False
 
-    def display(self) -> None:
+    def as_log(self) -> None:
+        """Logs all messages sorted by type and add a general summary"""
         out = []
         for message_list in self._message_map.values():
-            message_list.display()
+            message_list.as_log()
             if tmp := message_list.stats():
                 out.append(tmp)
 
         if out:
             logger.info(", ".join(out))
 
-    def get_as_text(self) -> str:
+    def as_text(self) -> str:
+        """Returns a concatenated string of all messages sorted by type"""
         return "\n".join(
             [
                 message.message
@@ -417,19 +406,22 @@ class LatexLogParser:
             ]
         )
 
-    def warning(self) -> LogMessageList:
+    def warning(self) -> LogMessageList[WarningMessage]:
+        """Returns WarningMessage"""
         return self._message_map[WarningMessage]
 
     def error(self) -> LogMessageList:
+        """Returns ErrorMessage"""
         return self._message_map[ErrorMessage]
 
     def latex3(self) -> LogMessageList:
+        """Returns Latex3Message"""
         return self._message_map[Latex3Message]
 
 
 class PdfLatex:
     class CompilationError(Exception):
-        ERRNO_MESSAGE = (66, 'Latex compilation error')
+        ERRNO_MESSAGE = (66, "Latex compilation error")
 
     class PdfCropError(Exception):
         ERRNO_MESSAGE = (64, "PDF crop error")
@@ -444,7 +436,7 @@ class PdfLatex:
         ERRNO_MESSAGE = (62, "Move error")
 
         def __init__(self, source: Path, dest: Path):
-            super().__init__(f'Moving {source} to {dest} failed')
+            super().__init__(f"Moving {source} to {dest} failed")
 
         @property
         def error(self) -> str:
@@ -460,13 +452,13 @@ class PdfLatex:
     @classmethod
     def compilation_command(cls, fullname: Path) -> list[str]:
         return [
-                "pdflatex",
-                "-shell-escape",
-                "-interaction=nonstopmode",
-                "-halt-on-error",
-                f"-output-directory={fullname.parent}",
-                str(fullname)
-            ]
+            "pdflatex",
+            "-shell-escape",
+            "-interaction=nonstopmode",
+            "-halt-on-error",
+            f"-output-directory={fullname.parent}",
+            str(fullname),
+        ]
 
     def __init__(self, fullname: Path):
         self._fullname = fullname.with_suffix(".tex")
@@ -477,20 +469,20 @@ class PdfLatex:
 
             llp = LatexLogParser(self._fullname, [WarningMessage])
             llp.process()
-            llp.display()
+            llp.as_log()
             if llp.should_recompile() and can_recompile:
                 logger.info("Automatic recompilation triggered")
                 self.compile()
                 llp = LatexLogParser(self._fullname, [WarningMessage])
                 llp.process()
-                llp.display()
+                llp.as_log()
                 logger.info("Automatic recompilation successful")
 
             self.move_pdf(dest)
         except self.CompilationError:
             llp = LatexLogParser(self._fullname, [Latex3Message, ErrorMessage])
             llp.process()
-            llp.display()
+            llp.as_log()
         except self.MoveError:
             logger.error(f"Moving pdf to {dest} failed")
 
@@ -500,7 +492,7 @@ class PdfLatex:
                 self.compilation_command(self._fullname),
                 capture_output=True,
                 check=True,
-                env=env
+                env=env,
             )
             logger.debug(f"{self._fullname.name}: pdflatex successful")
         except subprocess.CalledProcessError:
@@ -508,9 +500,7 @@ class PdfLatex:
             raise self.CompilationError
 
     def convert_pdf_to_png(
-        self,
-        dest_files: list[Path] = [],
-        cropped: bool = False
+        self, dest_files: list[Path] = [], cropped: bool = False
     ) -> None:
         if cropped:
             try:
@@ -520,7 +510,7 @@ class PdfLatex:
                         self._fullname.with_suffix(".pdf"),
                         self._fullname.with_suffix(".pdf"),
                     ],
-                    stderr=subprocess.STDOUT
+                    stderr=subprocess.STDOUT,
                 )
                 logger.debug(f"{self._fullname.name} PDF crop successful")
             except subprocess.CalledProcessError as e:
@@ -541,9 +531,9 @@ class PdfLatex:
                     self._fullname.with_suffix(".pdf"),
                     "-quality",
                     "90",
-                    self._fullname.with_suffix(".png")
+                    self._fullname.with_suffix(".png"),
                 ],
-                stderr=subprocess.STDOUT
+                stderr=subprocess.STDOUT,
             )
             logger.debug(f"{self._fullname.name}: PDF to PNG successful")
         except subprocess.CalledProcessError as e:
@@ -579,20 +569,3 @@ class PdfLatex:
                 shutil.move(self._fullname.with_suffix(".pdf"), dest)
             except Exception:
                 raise self.MoveError(self._fullname.with_suffix(".pdf"), dest)
-
-
-def texfile_to_symlink(tex_file: Path) -> Path:
-    fname = tex_file.resolve().with_suffix(".tex")
-    link = config.LATEX.aux
-    for p in fname.parent.parts[3:]:
-        link /= p
-    link /= fname.name
-
-    if link.exists():
-        return link
-
-    if not link.parent.exists():
-        link.parent.mkdir(parents=True, exist_ok=True)
-
-    link.symlink_to(fname)
-    return link
